using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WINDF7CSharp;

namespace Schwarze
{
    public partial class FormSchwarze : Form
    {
        FormMain meinMain;
        
        public FormSchwarze(FormMain m)
        {
            InitializeComponent();
            meinMain = m;

        }

        string t3010notfound = "t30/10 not found";
        string t7090notfound = "t70/90 not found";
        string onlyPT1modellposs = "only PT1 model possible";


        bool schwarzemethod_t30 = true;
        double t3 = 5;
        double t7 = 15;
        double t1 = 1;
        double t2 = 0.1;
        double tz = 0;

        double h2PT1(double t, double T1, double T2)
        {
            double ex1;
            double ex2;
            if (Math.Abs(T1) < 1e-10) T1 = 1e-10;
            if (Math.Abs(T2) < 1e-10) T2 = 1e-10;
            if (t / T1 > 80) ex1 = 0.0;
            else ex1 = Math.Exp(-t / T1);
            if (t / T2 > 80) ex2 = 0.0;
            else ex2 = Math.Exp(-t / T2);
            if (Math.Abs(T1 - T2) / T1 > 1e-10) return (1 + (-T1 * ex1 + T2 * ex2) / (T1 - T2));
            else return (1 - (T1 + t) / T1 * ex1);
        }

        double t30(double T1, double T2)
        {
            double tx = (T1 + T2) * 2.0;  /*start*/
            double intstart = 0;
            double intend = tx;
            double f;
            int i = 0;
            do
            {
                tx = intstart + (intend - intstart) / 2.0; /*tx in the middle of intervall */
                f = h2PT1(tx, T1, T2) - 0.3;
                if (f > 0) intend = tx;
                else intstart = tx;
                i = i + 1;
            }
            while (!((Math.Abs(f) < 1e-6) || (i > 30)));
            if (i > 30) MessageBox.Show(t3010notfound);
            else return (tx);
            return (0.0);
        }        /*t30*/

        double t70(double T1, double T2)
        {
            double tx = (T1 + T2) * 4.0;  /*start*/
            double intstart = 0;
            double intend = tx;
            double f;
            int i = 0;
            do
            {
                tx = intstart + (intend - intstart) / 2.0; /*tx in the middle of intervall */
                f = h2PT1(tx, T1, T2) - 0.7;
                if (f > 0) intend = tx;
                else intstart = tx;
                i = i + 1;
            }
            while (!((Math.Abs(f) < 1e-6) || (i > 30)));
            if (i > 30) MessageBox.Show(t7090notfound);
            else return (tx);
            return (0.0);
        }        /*t70*/

        double t10(double T1, double T2)
        {
            double tx = (T1 + T2) * 2.0;  /*start*/
            double intstart = 0;
            double intend = tx;
            double f;
            int i = 0;
            do
            {
                tx = intstart + (intend - intstart) / 2.0; /*tx in the middle of intervall */
                f = h2PT1(tx, T1, T2) - 0.1;
                if (f > 0) intend = tx;
                else intstart = tx;
                i = i + 1;
            }
            while (!((Math.Abs(f) < 1e-6) || (i > 30)));
            if (i > 30) MessageBox.Show(t3010notfound);
            else return (tx);
            return (0.0);
        }        /*t10*/

        double t90(double T1, double T2)
        {
            double tx = (T1 + T2) * 10.0;  /*start*/
            double intstart = 0;
            double intend = tx;
            double f;
            int i = 0;
            do
            {
                tx = intstart + (intend - intstart) / 2.0; /*tx in the middle of intervall */
                f = h2PT1(tx, T1, T2) - 0.9;
                if (f > 0) intend = tx;
                else intstart = tx;
                i = i + 1;
            }
            while (!((Math.Abs(f) < 1e-6) || (i > 30)));
            if (i > 30) MessageBox.Show(t7090notfound);
            else return (tx);
            return (0.0);
        }        /*t90*/

        void computeschwarze(double t3, double t7, ref double T1, ref double T2, ref double tz)
        {
          double x, a, b, aa, bb, hilf, x1, x2, f1, f2, t7i;
          int i;
          double[] powerx = new double[7];  //declaration of array
          tz = 0;
          T2 = 0;
          x = t3 / t7;
          if (x <= 0.2963 )
          {
            MessageBox.Show(onlyPT1modellposs);
            T1 = 0.8305835*t7;
          }
          else
            if (x < 0.317 )
            {
              T1 = 0.8305835 * t7; /* T1 of a PT1 */
              aa = Math.Log(0.7) / Math.Log(0.3);
              T2 = (t3 - aa * t7) / (1 - aa); /* T2 = t30 - (t30 of a PT1) */
            }
            else
              if (x > 0.43791054 )
              {
                //MessageBox.Show(bad2PT1approx);
                a = 0.4379105;
                tz = (t3 - a * t7) / (1 - a);
                T2 = 0.275950348 * (t7 - tz);
                T1 = 2 * T2;
              }
              else      /* switch (0.3021 < x < 0.43791) */
              {    
                powerx[0] = 1;

                for (i = 1 ; i <= 6; i++) powerx[i] = powerx[i - 1] * x;
                b = 197306.73613;
                b = b - 2969604.8601989 * powerx[1];
                b = b + 18607132.878200 * powerx[2];
                b = b - 62109381.452808 * powerx[3];
                b = b + 116456964.57735 * powerx[4];
                b = b - 116282883.73529 * powerx[5];
                b = b + 48300068.125023 * powerx[6];
                powerx[0] = 1;

                for (i = 1; i <= 6; i++) powerx[i] = powerx[i - 1] * b;
                hilf = 1.2839265375;
                hilf = hilf + 1.1606004314 * powerx[1];
                hilf = hilf + 2.6928736394e-3 * powerx[2];
                hilf = hilf - 8.0087808858e-5 * powerx[3];
                hilf = hilf + 1.2103880486e-6 * powerx[4];
                hilf = hilf - 8.9564553450e-9 * powerx[5];
                hilf = hilf + 2.5727300882e-11 * powerx[6];
                T2 = t7 / hilf;
                T1 = b * T2;
              }
              if ((x < 0.43791054) && (x > 0.2963))
              {
                //{ to improve accuracy one step with linear interpolation }
                /* improve T1 / T2 */
                x1 = t30(T1,T2) / t70(T1,T2);
                f1 = T1 / T2;   /* 1. point */
                x2 = t30(T1,T2 * 1.01) / t70(T1,T2 * 1.01);
                f2 = T1 / T2 / 1.01;  /* 2. point */
                aa = (f2 - f1) / (x2 - x1);
                bb = f1 - aa * x1;
                b = aa * x + bb;
                
                /* improve T7 / T2 */
                t7i = t70(T1,T2);
                x1 = t30(T1,T2) / t7i;
                f1 = t7i / T2;   /* 1. point */
                t7i = t70(T1,T2 * 1.01);
                x2 = t30(T1,T2 * 1.01) / t7i;
                f2 = t7i / T2 / 1.01;  /* 2. point */
                aa = (f2 - f1) / (x2 - x1);
                bb = f1 - aa * x1;
                hilf = aa * x + bb;
                T2 = t7 / hilf;
                T1 = b * T2;
              }
        }

        void computeschwarze10(double t3, double t7, ref double T1, ref double T2, ref double tz)
       /* t10/t90 method, code is identical, i <= t30 / t70, different numbers */
        {
          double x, a, b, aa, bb, hilf, x1, x2, f1, f2, t7i;
          int i;
          double[] powerx = new double[7];
          tz = 0;
          T2 = 0;
          x = t3 / t7;
          if (x <= 0.046)
          {
            MessageBox.Show(onlyPT1modellposs);
            T1 = 0.4342917 * t7;
          }
          else
            if (x > 0.1358 )
            {
              //MessageBox.Show(bad2PT1approx);
              a = 0.1319;
              tz = (t3 - a * t7) / (1 - a);
              T2 = 1.448 * (t3 - tz);
              T1 = 1.66667 * T2;
            }
            else    /* switch (0.046 < x < 0.1319 */
            {    
              powerx[0] = 1;
              x = x - 0.046;
              for (i = 1; i <= 6; i++) powerx[i] = powerx[i - 1] * x;
              b = 0.012104;
              b = b - 10.523378 * powerx[1];
              b = b + 2144.680341 * powerx[2];
              b = b - 120444.711795 * powerx[3];
              b = b + 2958416.872166 * powerx[4];
              b = b - 32602364.712813 * powerx[5];
              b = b + 132956037.989831 * powerx[6];
              powerx[0] = 1;
              hilf = 0.016173;
              hilf = hilf + 10.325519 * powerx[1];
              hilf = hilf + 1516.662283 * powerx[2];
              hilf = hilf - 100785.196041 * powerx[3];
              hilf = hilf + 2608057.096111 * powerx[4];
              hilf = hilf - 29449395.311493 * powerx[5];
              hilf = hilf + 121894706.637873 * powerx[6];
              T2 = t3 * hilf;
              T1 = T2 / b;
              x = x + 0.046;
            }
          if ((x < 0.1358) && (x > 0.046))
          {
            // { to improve accuracy one step with linear interpolation }
            /* improve T1 / T2 */
            x1 = t10(T1,T2) / t90(T1,T2);
            f1 = T1 / T2;   /* 1. point */
            x2 = t10(T1,T2 * 1.01) / t90(T1,T2 * 1.01);
            f2 = T1 / T2 / 1.01;  /* 2. point */
            aa = (f2 - f1) / (x2 - x1);
            bb = f1 - aa * x1;
            b = aa * x + bb;

            /*improve T7/T2 */
            t7i = t90(T1,T2);
            x1 = t10(T1,T2) / t7i;
            f1 = t7i / T2;   /* 1. point */
            t7i = t90(T1,T2 * 1.01);
            x2 = t10(T1,T2 * 1.01) / t7i;
            f2 = t7i / T2 / 1.01;  /* 2. point */
            aa = (f2 - f1) / (x2 - x1);
            bb = f1 - aa * x1;
            hilf = aa * x + bb;
            T2 = t7 / hilf;
            T1 = b * T2;
          }
        }

//--------------------------------------------------------------------//

        private void Form1_Activated(object sender, EventArgs e)
        {
         
        }

        private void BitBtn2_Click(object sender, EventArgs e)
        {
            String a = "      ",crlf="\x0d" + "\x0a";
            meinMain.tBMainReport.AppendText("\r\n");
            meinMain.tBMainReport.AppendText(a + "_____________________________________________________\r\n");
            meinMain.tBMainReport.AppendText("\r\n");
            meinMain.tBMainReport.AppendText(a + Text + crlf);// Headline
            meinMain.tBMainReport.AppendText(a + crlf);
            if (radioButton1.Checked)
                meinMain.tBMainReport.AppendText(a + label5.Text + "  t30/70" + crlf);
            if (radioButton2.Checked)
                meinMain.tBMainReport.AppendText(a + label5.Text + "  t10/90" + crlf);
            meinMain.tBMainReport.AppendText(a + crlf);
            meinMain.tBMainReport.AppendText(a + label6.Text + " = " + textBox2.Text + "   " + label13.Text + crlf);
            meinMain.tBMainReport.AppendText(a + label7.Text + " = " + textBox3.Text + "   " + label14.Text + crlf);
            meinMain.tBMainReport.AppendText(a + label11.Text + " = " + textBox6.Text + crlf);
            meinMain.tBMainReport.AppendText(a + crlf);
            meinMain.tBMainReport.AppendText(a + label9.Text + " = " + textBox4.Text + "   " + label15.Text + crlf);
            meinMain.tBMainReport.AppendText(a + label10.Text + " = " + textBox5.Text + "   " + label16.Text + crlf);
            meinMain.tBMainReport.AppendText(a + label12.Text + " = " + textBox7.Text + crlf);


            meinMain.tBMainReport.AppendText(a + crlf);
            meinMain.tBMainReport.AppendText(a + "_____________________________________________________" + crlf);
        }

        public void deutsch()
        {
            Text = "Identifikation eines 2PT1-Modelles";
            BitBtn1.Text = "Schlieen";
            label1.Text = "Identifikation";
            label4.Text = "eines 2PT1-Modelles";
            label2.Text = "mit der Methode von";
            label3.Text = "Schwarze";
            label12.Text = "Totzeit";
            button1.Text = "&Empfindlichkeit";
            label5.Text = "Methode";
            BitBtn2.Text = "&zum Report";
            BitBtn1.Text = "&Abbruch";
            toolTip1.SetToolTip(BitBtnOK, "kopiere Ergebnisse zu den Reglerentwurfsfenstern");
            toolTip1.SetToolTip(BitBtn1, "Schliee Fenster und ignoriere Ergebnisse");

            onlyPT1modellposs = "Nur PT1 Modell mglich";
            t3010notfound = "t30/10 nicht gefunden";
            t7090notfound = "t70/90 nicht gefunden";
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
             
            if (radioButton1.Checked)
            {
                label6.Text = "t30";
                label7.Text = "t70";
                label11.Text = "t30/t70";
                //    schwarzemethod = "t30";
                schwarzemethod_t30 = true;
            }
            else
            {
                label6.Text = "t10";
                label7.Text = "t90";
                label11.Text = "t10/t90";
                //    schwarzemethod = "t10";
                schwarzemethod_t30 = false;
            }
            textBox3_Leave(sender, e);
          
        }

        private void textBox3_Leave(object sender, EventArgs e)
        {
            double x;
            meinMain.ConvertToDouble(textBox2.Text,ref t3 );
            meinMain.ConvertToDouble(textBox3.Text,ref t7 );
            x = t3 / t7;
            textBox6.Text = (x).ToString("G10");
            if (schwarzemethod_t30) computeschwarze(t3, t7, ref t1, ref t2, ref tz);
            else computeschwarze10(t3, t7, ref t1, ref t2, ref tz);
            if (tz == 0) textBox7.BackColor = Color.WhiteSmoke;
                    else textBox7.BackColor = Color.LightSalmon;
            textBox4.Text = (t1).ToString("G10");
            textBox5.Text = (t2).ToString("G10");
            textBox7.Text = (tz).ToString("G10");
            label13.Text = ""; /*clear sensitivities */
            label14.Text = "";
            label15.Text = "";
            label16.Text = "";
        }

        private void textBox4_Leave(object sender, EventArgs e)
        {
            meinMain.ConvertToDouble(textBox4.Text,ref t1 );
            meinMain.ConvertToDouble(textBox5.Text,ref t2 );
            if (schwarzemethod_t30)
            {
                t3 = t30(t1, t2) + tz;
                t7 = t70(t1, t2) + tz;
            }
            else
            {
                t3 = t10(t1, t2) + tz;
                t7 = t90(t1, t2) + tz;
            }
            textBox2.Text = (t3).ToString("G10");
            textBox3.Text = (t7).ToString("G10");
            textBox6.Text = (t3 / t7).ToString("G10");
            label13.Text = ""; /*clear sensitivities */
            label14.Text = "";
            label15.Text = "";
            label16.Text = "";
        }

        private void button1_Click(object sender, EventArgs e)
        {
            double t1 = 1e10, t2 = 0, tz = 0, dt1, dt2, t1n = 0, t2n = 0, dt1max, dt2max;
            /* sensitivity */
            meinMain.ConvertToDouble(textBox2.Text,ref t3 );
            meinMain.ConvertToDouble(textBox3.Text,ref t7 );
            label13.Text = "1%";
            label14.Text = "1%";
            if (schwarzemethod_t30) computeschwarze(t3, t7, ref t1, ref t2, ref tz);
            else computeschwarze10(t3, t7, ref t1, ref t2, ref tz);
            t3 = t3 * 1.01;
            if (schwarzemethod_t30) computeschwarze(t3, t7, ref t1n, ref t2n, ref tz);
            else computeschwarze10(t3, t7, ref t1n, ref t2n, ref tz);
            dt1max = Math.Abs((t1 - t1n) / t1 * 100);
            dt2max = Math.Abs((t2 - t2n) / t2 * 100);
            t3 = t3 / 1.01;
            t7 = t7 / 1.01;
            if (schwarzemethod_t30) computeschwarze(t3, t7, ref t1n, ref t2n, ref tz);
            else computeschwarze10(t3, t7, ref t1n, ref t2n, ref tz);
            dt1 = Math.Abs((t1 - t1n) / t1 * 100); dt2 = Math.Abs((t2 - t2n) / t2 * 100);
            t7 = t7 * 1.01;
            if (dt1 > dt1max) dt1max = dt1;
            if (dt2 > dt2max) dt2max = dt2;
            label15.Text = (dt1max).ToString("G4") + "%";
            label16.Text = (dt2max).ToString("G4") + "%";
        }

        private void BitBtn1_Click(object sender, EventArgs e)
        {
            this.Hide();
        }

        private void BitBtnOK_Click(object sender, EventArgs e)
        {
            this.Hide();
            meinMain.schwarzeOK();

        }

        private void textBox2_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 0x0d) textBox3_Leave(sender, e);
        }

        private void textBox4_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 0x0d) textBox4_Leave(sender, e);

        }

        private void FormSchwarze_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Visible = false;
        }

    }
}

