using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

namespace DFUFK
{
    public class StringGrid : System.Windows.Forms.DataGrid
    {
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;
        private System.Data.DataTable mDataTable;

        public StringGrid()
        {
            InitializeComponent();
            mDataTable = new DataTable("ParentTable");
        }

        public void addColumn(System.Type fColType, string fColName, bool
        fReadonly)
        {
            // Declare variables for DataColumn and DataRow objects.
            DataColumn aDataColumn;

            aDataColumn = new DataColumn();
            aDataColumn.DataType = fColType;
            aDataColumn.ColumnName = fColName;
            aDataColumn.ReadOnly = fReadonly;
            mDataTable.Columns.Add(aDataColumn);
        }


        public void activate()
        {
            // Instantiate the DataSet variable.
            DataSet myDataSet = new DataSet();
            // Add the new DataTable to the DataSet.
            myDataSet.Tables.Add(mDataTable);

            this.DataSource = this.mDataTable;
        }
        /// <summary>
        /// remove the selected row(record) from the grid
        /// </summary>
        public void RemoveSelectedRow()
        {
            mDataTable.Rows.RemoveAt(this.CurrentRowIndex);
            mDataTable.AcceptChanges();
        }
        /// <summary>
        /// add a new datarow to the string grid
        /// </summary>
        /// <param name="fNewRow">the new dataRow</param>
        public void AddRow(DataRow fNewRow)
        {
            mDataTable.Rows.Add(fNewRow);
        }
        /// <summary>
        /// exposing the dataTable's newRow method
        /// </summary>
        /// <returns></returns>
        public DataRow NewRow()
        {
            return mDataTable.NewRow();
        }
        /// <summary>
        /// replace the requested row with a new row
        /// </summary>
        /// <param name="fNewRow">the new row to put insted the old
        /// <param name="fIndex">the index of the replaced row</param>
        public void ReplaceAt(DataRow fNewRow, int fIndex)
        {
            this.mDataTable.Rows.RemoveAt(fIndex);
            this.mDataTable.Rows.InsertAt(fNewRow, fIndex);
            this.mDataTable.AcceptChanges();
        }
        /// <summary>
        /// replace the current selected row
        /// </summary>
        /// <param name="fNewRow">the new row</param>
        public void ReplaceAtSelected(DataRow fNewRow)
        {
            ReplaceAt(fNewRow, this.CurrentRowIndex);
        }
        public void InsertAt(DataRow fRow, int fIndex)
        {
            if (fIndex <= -1)
                fIndex = 0;
            mDataTable.Rows.InsertAt(fRow, fIndex);
            mDataTable.AcceptChanges();
        }
        /// <summary>
        /// returns the number of rows in the string grid
        /// </summary>
        /// <returns>the number of rows in the string grid</returns>
        public int RowCount
        {
            get { return this.mDataTable.Rows.Count; }
        }
        /// <summary>
        /// clear all rows from the string grid
        /// </summary>
        public void ClearAll()
        {
            this.mDataTable.Rows.Clear();
        }
        /// <summary>
        /// get the cell content
        /// </summary>
        /// <param name="fRow">the cell's row</param>
        /// <param name="fColumn">the cell's column</param>
        /// <returns></returns>
        public object Cell(int fRow, int fColumn)
        {
            return this.mDataTable.Rows[fRow][fColumn];
        }

        public void Cell(int fRow, int fColumn, string data)
        {
            this.mDataTable.Rows[fRow][fColumn] = data;
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            //
            // StringGrid
            //
            this.Name = "StringGrid";
            this.Size = new System.Drawing.Size(296, 224);

        }
    }
}